Package: MMRcaseselection
Type: Package
Title: Case Classification and Selection Based on Regression Results
Version: 0.2.0
Authors@R: 
    person("Ingo", "Rohlfing", email = "ingo.rohlfing@uni-passau.de",
           role = c("aut", "cre"),
           comment = c(ORCID = "0000-0001-8715-4771"))
Depends: R (>= 2.10)
Description: Researchers doing a mixed-methods analysis (nested analysis as
    developed by Lieberman (2005) <doi:10.1017/S0003055405051762>) can
    use the package for the classification of cases and case selection using
    results of a linear regression. One can designate cases 
    as typical, deviant, extreme and pathway case and use different case 
    selection strategies for the choice of a case belonging to one of
    these types.
License: GPL-3
Encoding: UTF-8
Imports: stats, ggplot2
URL: https://github.com/ingorohlfing/MMRcaseselection
BugReports: https://github.com/ingorohlfing/MMRcaseselection/issues
Suggests: knitr, rmarkdown
VignetteBuilder: knitr
Language: en-US
Config/roxygen2/version: 8.0.0
NeedsCompilation: no
Packaged: 2026-05-17 17:35:46 UTC; ingor
Author: Ingo Rohlfing [aut, cre] (ORCID:
    <https://orcid.org/0000-0001-8715-4771>)
Maintainer: Ingo Rohlfing <ingo.rohlfing@uni-passau.de>
Repository: CRAN
Date/Publication: 2026-05-21 09:32:15 UTC
Built: R 4.5.2; ; 2026-05-25 01:32:14 UTC; unix

Package: MMRcaseselection
Type: Package
Title: Case Classification and Selection Based on Regression Results
Version: 0.2.0
Authors@R: 
    person("Ingo", "Rohlfing", email = "ingo.rohlfing@uni-passau.de",
           role = c("aut", "cre"),
           comment = c(ORCID = "0000-0001-8715-4771"))
Depends: R (>= 2.10)
Description: Researchers doing a mixed-methods analysis (nested analysis as
    developed by Lieberman (2005) <doi:10.1017/S0003055405051762>) can
    use the package for the classification of cases and case selection using
    results of a linear regression. One can designate cases 
    as typical, deviant, extreme and pathway case and use different case 
    selection strategies for the choice of a case belonging to one of
    these types.
License: GPL-3
Encoding: UTF-8
Imports: stats, ggplot2
URL: https://github.com/ingorohlfing/MMRcaseselection
BugReports: https://github.com/ingorohlfing/MMRcaseselection/issues
Suggests: knitr, rmarkdown
VignetteBuilder: knitr
Language: en-US
Config/roxygen2/version: 8.0.0
NeedsCompilation: no
Packaged: 2026-05-17 17:35:46 UTC; ingor
Author: Ingo Rohlfing [aut, cre] (ORCID:
    <https://orcid.org/0000-0001-8715-4771>)
Maintainer: Ingo Rohlfing <ingo.rohlfing@uni-passau.de>
Repository: CRAN
Date/Publication: 2026-05-21 09:32:15 UTC
Built: R 4.6.0; ; 2026-05-25 03:35:20 UTC; unix

# Generated by roxygen2: do not edit by hand

export(extreme_on_x)
export(extreme_on_y)
export(most_deviant)
export(most_overpredicted)
export(most_typical)
export(most_underpredicted)
export(pathway)
export(pathway_xvr)
export(predint)
export(predint_plot)
export(residstd)
export(residstd_plot)
import(ggplot2)
importFrom(stats,lm)
importFrom(stats,predict.lm)
importFrom(stats,residuals)

# MMRcaseselection <img src="man/figures/MMRhex.png" height="200" align="right">

[![CRAN
status](http://www.r-pkg.org/badges/version/MMRcaseselection)](https://cran.r-project.org/package=MMRcaseselection)
[![Project Status: Active – The project has reached a stable, usable state and is being actively developed.](https://www.repostatus.org/badges/latest/active.svg)](https://www.repostatus.org/#active)
[![GPLv3 License](https://img.shields.io/badge/License-GPL%20v3-yellow.svg)](https://opensource.org/license)
![](https://cranlogs.r-pkg.org/badges/grand-total/MMRcaseselection)

## Overview
Classification and choice of cases for case studies based on regression results.

For now, the package contains functions for the classification of cases as 
typical, deviant, extreme or pathway cases and the choice of these types of cases. 
At this stage, the focus is on single cases and case selection based on linear regression models (class `lm`).

The package is [available on CRAN](https://cran.r-project.org/package=MMRcaseselection). 
The version on CRAN Github is 0.2.0. The package can be installed from CRAN with the following command.

```r
install.packages("MMRcaseselection")
```

The package can be installed from Github with the following command.

```r
pak::pak("ingorohlfing/MMRcaseselection")
```

The documentation of the package and examples can be found 
[here](https://ingorohlfing.github.io/MMRcaseselection/docs/). Please open an
issue on Github if problems come up.

***

Work on the package was funded by the European Research Council (ERC) under the 
European Union’s Horizon 2020 research and innovation program (grant agreement 
number 638425, *Enhanced Qualitative and Multimethod Research*).

#' Pathway case
#'
#' Calculation of pathway values, defined as the difference between residuals of
#' full model and reduced model lacking the pathway variable. The larger the
#' difference, the more a case qualifies as a pathway case suitable for the
#' analysis of mechanisms.
#'
#' The difference between the absolute residuals of the full and reduced model
#' follows the approach developed by Weller and Barnes (2014): \emph{Finding
#' Pathways: Mixed-Method Research for Studying Causal Mechanisms.}
#' Cambridge: Cambridge University Press.
#' \doi{10.1017/CBO9781139644501}).
#'
#' The calculation of the absolute difference between the full-model and
#' reduced-model residuals, given a case's reduced-model residual is larger
#' than its full-model residual, follows the proposal by
#' Gerring (2007): Is There a (Viable) Crucial-Case Method?
#' \emph{Comparative Political Studies} 40 (3): 231-253.
#' \doi{10.1177/0010414006290784})
#'
#' @param full_model Full model including covariate of interest
#' (= pathway variable)
#' @param reduced_model Reduced model excluding covariate of interest
#'
#' @return A dataframe with
#'
#' - all full model variables,
#'
#' - full model residuals (\code{full_resid}),
#'
#' - reduced model residuals (\code{reduced_resid}),
#'
#' - pathway values following Weller/Barnes (\code{pathway_wb}),
#'
#' - pathway values following Gerring (\code{pathway_gvalue}),
#'
#' - variable showing whether Gerring's criterion for a pathway
#' case is met (\code{pathway_gstatus})
#'
#' @importFrom stats lm residuals
#'
#' @examples
#' df_full <- lm(mpg ~ disp + wt, data = mtcars)
#' df_reduced <- lm(mpg ~ wt, data = mtcars)
#' pathway(df_full, df_reduced)
#'
#' @export
pathway <- function(full_model, reduced_model) {
  if (inherits(full_model, "lm")) {
    if (inherits(reduced_model, "lm")) {
      # full model
      full_resid <- residuals(full_model)
      # reduced model
      reduced_resid <- residuals(reduced_model)
      # difference between absolute residuals
      pathway_wb <- abs(reduced_resid) - abs(full_resid)
      # absolute difference between residuals
      pathway_gvalue <- abs(reduced_resid - full_resid)
      # check for Gerring's criterion for pathway values
      pathway_gtype <- ifelse(abs(reduced_resid) > abs(full_resid), "yes", "no")
      comb <- cbind(full_model$model, full_resid, reduced_resid,
                    pathway_wb, pathway_gvalue, pathway_gtype)
      return(comb)
    }
    else{
      stop("Reduced model object is not of class lm")
    }
  }
  else{
    (stop("Full model object is not of class lm"))
  }
}

#' Plot of residuals against pathway variable
#'
#' @param full_model Full model including covariate of interest
#' (= pathway variable)
#' @param reduced_model Reduced model excluding covariate of interest
#' @param pathway_type Type of pathway values. \code{pathway_wb} are
#' pathway values proposed by Weller and Barnes. \code{pathway_gvalue}
#' are values as calculated by Gerring.
#'
#' @return A plot of the chosen type of pathway values against the pathway
#' variable created with \pkg{ggplot2}.
#'
#' @import ggplot2
#'
#' @examples
#' df_full <- lm(mpg ~ disp + wt, data = mtcars)
#' df_reduced <- lm(mpg ~ wt, data = mtcars)
#' pathway_xvr(df_full, df_reduced, pathway_type = "pathway_wb")
#'
#' @export
pathway_xvr <- function(full_model, reduced_model, pathway_type) {
  pwdf <- pathway(full_model, reduced_model)
  if (pathway_type == "pathway_wb") {
    pwplot <- ggplot2::ggplot() +
      geom_point(data = pwdf,
                 mapping = aes_string(x = setdiff(names(full_model$model),
                                                  names(reduced_model$model)),
                                      y = pathway_type)) +
      geom_hline(yintercept = 0, linetype = 5) +
      scale_y_continuous("Pathway values") +
      theme_classic() -> pwplot
  }
  else{
    pwplot <- ggplot2::ggplot() +
      geom_point(data = pwdf,
                 mapping = aes_string(x = setdiff(names(full_model$model),
                                                  names(reduced_model$model)),
                                      y = pathway_type,
                                      color = "pathway_gtype")) +
      geom_hline(yintercept = 0, linetype = 5) +
      scale_y_continuous("Pathway values") +
      scale_color_viridis_d("Reduced > full residuals") +
      theme_classic() +
      theme(legend.position = "bottom") -> pwplot
  }
  return(pwplot)
}

#' Identification of the most typical case
#'
#' The most typical case (= best predicted case) based on regression estimates.
#'
#' Proposed by Seawright, Jason and John Gerring (2008):
#' Case Selection Techniques in Case Study Research: A Menu of
#' Qualitative and Quantitative Options. \emph{Political Research Quarterly}
#' 61 (2): 294-308.
#' (\doi{10.1177/1065912907313077})
#'
#' @param lmobject Object generated with \code{\link[stats]{lm}}
#'
#' @return The most typical case having the smallest absolute
#' residual of all cases.
#'
#' @importFrom stats lm residuals
#'
#' @examples
#' df <- lm(mpg ~ disp + wt, data = mtcars)
#' most_typical(df)
#'
#' @export
most_typical <- function(lmobject) {
  if (inherits(lmobject, "lm")) {
  absresid <- sort(abs(residuals(lmobject)))
  return(absresid[1])
  }
  else{
    stop("Input into function is not of class lm")
  }
}

#' Classification of cases as typical and deviant using a prediction
#' interval.
#'
#' Case are designated as typical (= well predicted) and deviant
#' (= badly predicted) based on the prediction interval. The x\% prediction
#' interval represents the range that we expect to include
#' x\% of outcome values in repeated samples. For example, a 95\%
#' prediction interval ranging from 0-5 conveys that 95\% of future outcome
#' values will be in the range of 0-5. If the observed outcome
#' is inside the prediction interval, the case is classified (or designated) as
#'  typical and as deviant otherwise.
#'
#' Proposed by Rohlfing, Ingo and Peter Starke (2013):
#' Building on Solid Ground: Robust Case Selection in Multi-Method Research.
#' *Swiss Political Science Review* 19 (4): 492-512.
#' (\doi{10.1111/spsr.12052})
#'
#' @param lmobject Object generated with \code{\link[stats]{lm}}
#' @param piwidth Width of the prediction interval (default is 0.95).
#'
#' @return A dataframe with the observed outcome, fitted outcome,
#' upper and lower bound of the \% prediction interval and classification
#' of cases as typical or deviant.
#'
#' @importFrom stats lm residuals predict.lm
#'
#' @examples
#' df <- lm(mpg ~ disp + wt, data = mtcars)
#' predint(df, piwidth = 0.9)
#'
#' @export
predint <- function(lmobject, piwidth = 0.95) {
  if (inherits(lmobject, "lm")) {
    if (piwidth >= 0 & piwidth <= 1) {
      # calculating prediction interval
      temp <- as.data.frame(suppressWarnings(predict.lm(lmobject,
                                                        interval = "prediction",
                                                        level = piwidth)))
      # extracting outcome values
      outcome <- lmobject$model[, 1]
      # merging outcome values into dataframe
      comb <- cbind(temp, outcome)
      # classification of cases
      comb$status <- ifelse(comb$outcome < comb$lwr |
                              comb$outcome > comb$upr,
                            "deviant", "typical")
      return(comb)
    }
    else{
      stop("Prediction interval needs to be between 0 and 1")
    }
  }
  else{
    stop("Input into function is not of class lm")
  }
}

#' Plot of typical and deviant cases with prediction intervals
#'
#' Presented in Rohlfing, Ingo and Peter Starke (2013):
#' Building on Solid Ground: Robust Case Selection in Multi-Method Research.
#' \emph{Swiss Political Science Review} 19 (4): 492-512.
#' (\doi{10.1111/spsr.12052})
#'
#' @param pred_df A dataframe created with \code{\link{predint}}.
#'
#' @return A plot of the observed outcome against the fitted outcome with
#' prediction intervals and case classifications. Created with
#' \pkg{ggplot2}.
#'
#' @import ggplot2
#
#' @examples
#' df <- lm(mpg ~ disp + wt, data = mtcars)
#' predint_status <- predint(df, piwidth = 0.9)
#' predint_plot(predint_status)
#'
#' @export
predint_plot <- function(pred_df) {
  ggplot(data = pred_df) +
    geom_point(mapping = aes(x = fit, y = outcome, color = status)) +
    # bisecting line
    geom_abline(intercept = 0, slope = 1, linetype = 5) +
    # prediction intervals
    geom_errorbar(mapping = aes(x = fit, ymin = lwr, ymax = upr,
                                color = status)) +
    # colorblind scheme
    scale_color_viridis_d() +
    theme_classic() +
    theme(legend.title = element_blank())
}

#' Classification of cases as typical and deviant using the standard
#' deviation of the residuals.
#'
#' The share of the standard deviation of the residuals is used to
#' designate cases as typical or deviant.
#'
#' Proposed by Lieberman, Evan S. (2005): Nested Analysis as a Mixed-Method
#' Strategy for Comparative Research. \emph{American Political Science Review}
#' 99 (3): 435-452. \doi{10.1017/S0003055405051762}.
#'
#' @param lmobject Object generated with \code{\link[stats]{lm}}
#' @param stdshare Share of standard deviation of residuals distinguishing
#' between typical and deviant cases (default is 1).
#'
#' @return A dataframe with the observed outcome, fitted outcome,
#' residual standard deviation and classification of cases as typical
#' or deviant.
#'
#' @importFrom stats lm residuals
#'
#' @examples
#' df <- lm(mpg ~ disp + wt, data = mtcars)
#' residstd(df, stdshare = 1)
#'
#' @export
residstd <- function(lmobject, stdshare = 1) {
  if (inherits(lmobject, "lm")) {
    if (stdshare >= 0) {
      # calculating standard deviation of residuals
      tempsd <- as.data.frame(suppressWarnings(predict.lm(df, se.fit = T)))
      # removing irrelevant columns
      tempsd <- tempsd[, c("fit", "residual.scale")]
      # extracing outcome values
      outcome <- df$model[, 1]
      # merging outcome values into dataframe
      comb <- cbind(tempsd, outcome)
      # classification of cases
      comb$status <- ifelse(comb$outcome <
                              comb$fit - stdshare * comb$residual.scale |
                              comb$outcome >
                              comb$fit + stdshare * comb$residual.scale,
                            "deviant", "typical")
      return(comb)
      }
    else{
      stop("Standard deviation should not be negative")
    }
  }
  else{
    stop("Input into function is not of class lm")
  }
}

#' Plot of typical and deviant cases based on residuals' standard deviation
#'
#' @param resid_df A dataframe created with \code{\link{residstd}}.
#'
#' @return A plot of the observed outcome against the fitted outcome with
#' interval and case classifications. Created with \pkg{ggplot2}.
#'
#' @import ggplot2
#'
#' @examples
#' df <- lm(mpg ~ disp + wt, data = mtcars)
#' residstd_status <- residstd(df, stdshare = 1)
#' residstd_plot(residstd_status)
#'
#' @export
residstd_plot <- function(resid_df) {
  # Calculation of upper and lower bounds
  resid_df$lwr <- resid_df$fit - resid_df$residual.scale
  resid_df$upr <- resid_df$fit + resid_df$residual.scale
  ggplot(data = resid_df) +
    # lower bound
    geom_line(mapping = aes(x = fit, y = lwr), color = "grey") +
    # upper bound
    geom_line(mapping = aes(x = fit, y = upr), color = "grey") +
    geom_point(mapping = aes(x = fit, y = outcome, color = status),
               size = 2) +
    # bisecting line
    geom_abline(intercept = 0, slope = 1, linetype = 5) +
    # colorblind scheme
    scale_color_viridis_d() +
    theme_classic() +
    theme(legend.title = element_blank())
}

#' Extremeness of cases on an independent variable
#'
#' Extremeness of a case is calculated by the difference between a case's
#' value on the independent variable and the variable's mean value.
#'
#' Calculating the absolute value of the difference between the cases' values
#' and the variable's mean value is proposed by Seawright, Jason (2016): The
#' Case for Selecting Cases That Are Deviant or Extreme on the Independent
#' Variable. \emph{Sociological Methods & Research} 45 (3): 493-525.
#' (\doi{10.1177/0049124116643556})
#'
#' @param lmobject Object generated with \code{\link[stats]{lm}}
#' @param ind_var Independent variable for which extremeness values should
#' be calculated. Has to be entered as a character.
#'
#' @return A dataframe with
#'
#' - all variables in the linear model,
#'
#' - absolute extremeness (absolute value of difference between variable
#' score and mean value of variable),
#'
#' - extremeness (difference between variable score and mean value of
#' variable), which can be useful when the direction of extremeness is relevant.
#'
#' The rows are ordered in decreasing order of the absolute extreme values.
#'
#' @importFrom stats lm
#'
#' @examples
#' df <- lm(mpg ~ disp + wt, data = mtcars)
#' extreme_on_x(df, "wt")
#'
#' @export
extreme_on_x <- function(lmobject = NULL, ind_var = NULL) {
  if (is.null(ind_var)) {
    stop("Please specify the independent variable")
  }
  if (!inherits(lmobject, "lm")) {
    stop("lmobject input into function is not of class lm")
  }
  if (isFALSE(ind_var %in% names(lmobject$model[, 2:ncol(lmobject$model)]))) {
    stop("Chosen ind_var is not among the independent variables")
  }
  else{
    tempx <- lmobject$model[, ind_var]
    tempdf <- lmobject$model
    tempdf$`abs. extremeness` <- abs(tempx - mean(tempx))
    tempdf$extremeness <- tempx - mean(tempx)
    tempdf <- tempdf[order(tempdf$`abs. extremeness`, decreasing = T), ]
    return(tempdf)
  }
}

#' Extremeness of cases on the dependent variable
#'
#' Extremeness of a case is calculated by the difference between a case's
#' value on the dependent variable and the variable's mean value.
#'
#' Calculating the absolute value of the difference between the cases' values
#' and the variable's mean value is proposed by Seawright, Jason (2016): The
#' Case for Selecting Cases That Are Deviant or Extreme on the Independent
#' Variable. \emph{Sociological Methods & Research} 45 (3): 493-525.
#' (\doi{10.1177/0049124116643556})
#'
#' @param lmobject Object generated with \code{\link[stats]{lm}}
#'
#' @return A dataframe with
#'
#' - all variables in the linear model,
#'
#' - absolute extremeness (absolute value of difference between variable
#' score and mean value of variable),
#'
#' - extremeness (difference between variable score and mean value of
#' variable), which can be useful when the direction of extremeness is relevant.
#'
#' The rows are ordered in decreasing order of the absolute extreme values.
#'
#' @examples
#' df <- lm(mpg ~ disp + wt, data = mtcars)
#' extreme_on_y(df)
#'
#' @importFrom stats lm
#'
#' @export
extreme_on_y <- function(lmobject) {
  if (inherits(lmobject, "lm")) {
    tempy <- lmobject$model[, 1]
    tempdf <- lmobject$model
    tempdf$`abs. extremeness` <- abs(tempy - mean(tempy))
    tempdf$extremeness <- tempy - mean(tempy)
    tempdf <- tempdf[order(tempdf$`abs. extremeness`, decreasing = T), ]
    return(tempdf)
  }
  else{
    stop("lmobject input into function is not of class lm")
  }
}

#' Identification of the most deviant case
#'
#' Identification of the most deviant case (= worst predicted case),
#' based on regression estimates.
#'
#' Proposed by Seawright, Jason and John Gerring (2008):
#' Case Selection Techniques in Case Study Research: A Menu of
#' Qualitative and Quantitative Options. \emph{Political Research Quarterly}
#' 61 (2): 294-308.
#' (\doi{10.1177/1065912907313077})
#'
#' @param lmobject Object generated with \code{\link[stats]{lm}}
#'
#' @return The most deviant case with the largest absolute
#' residual of all cases.
#'
#' @importFrom stats lm residuals
#'
#' @examples
#' df <- lm(mpg ~ disp + wt, data = mtcars)
#' most_deviant(df)
#' @export
most_deviant <- function(lmobject) {
  if (inherits(lmobject, "lm")) {
    absresid <- sort(abs(residuals(lmobject)))
    return(absresid[length(absresid)])
  }
  else{
    stop("Input into function is not of class lm")
  }
}

#' Identification of the most overpredicted case
#'
#' The case with the largest negative difference between the observed
#' value and the predicted value on the outcome. Depending on the research
#' question, there might be a specific interest in the case for which the
#' model performs worst and yields a larger predicted value.
#'
#' @param lmobject Object generated with \code{\link[stats]{lm}}
#'
#' @return The most overpredicted case with the largest negative residual
#' (the most negative residual).
#' @importFrom stats lm residuals
#' @examples
#' df <- lm(mpg ~ disp + wt, data = mtcars)
#' most_overpredicted(df)
#' @export
most_overpredicted <- function(lmobject) {
  if (inherits(lmobject, "lm")) {
    resid <- sort(residuals(lmobject))
    return(resid[1])
  }
  else{
    stop("Input into function is not of class lm")
  }
}

#' Identification of the most underpredicted case
#'
#' The case with the largest positive difference between the observed
#' value and the predicted value on the outcome. Depending on the research
#' question, there might be a specific interest in the case for which the
#' model performs worst and yields a smaller predicted value.
#'
#' @param lmobject Object generated with \code{\link[stats]{lm}}
#'
#' @return The most underpredicted case with the largest positive residual
#' (the most positive residual).
#'
#' @importFrom stats lm residuals
#'
#' @examples
#' df <- lm(mpg ~ disp + wt, data = mtcars)
#' most_underpredicted(df)
#'
#' @export
most_underpredicted <- function(lmobject) {
  if (inherits(lmobject, "lm")) {
    resid <- sort(residuals(lmobject))
    return(resid[length(resid)])
  }
  else{
    stop("Input into function is not of class lm")
  }
}

---
title: "Most typical and most deviant"
author: "Ingo Rohlfing"
date: "`r Sys.Date()`"
output: rmarkdown::html_vignette
vignette: >
  %\VignetteIndexEntry{Most typical and most deviant}
  %\VignetteEngine{knitr::rmarkdown}
  %\VignetteEncoding{UTF-8}
---

```{r, include = FALSE}
knitr::opts_chunk$set(
  collapse = TRUE,
  comment = "#>"
)
```

```{r setup, message = F}
library(MMRcaseselection)
```

One can use four functions to choose four *types of cases* without classifying all cases as either typical or deviant. The *most typical* and *most deviant* cases are proposed by Seawright and Gerring ([2008](https://doi.org/10.1177/1065912907313077)). The most typical case has the smallest residual of all cases. The most deviant case has the largest residual of all cases. The two functions `most_typical()` and `most_deviant()` work in the same way and show you the case with its residual. The input into the function is an `lm` object.
```{r}
df <- lm(mpg ~ disp + wt, data = mtcars)
most_typical(df)
most_deviant(df)
```

The most deviant case does not distinguish between cases that have a large negative and a large positive residual. Cases with a negative residual are *overpredicted* because the predicted outcome is higher than the observed outcome. Cases with a positive residual are *underpredicted* because the predicted outcome is lower than the observed outcome. It might not matter whether a case is overpredicted or underpredicted because both subtypes of outliers can have the same type of deviance. However, one might be interested in knowing whether a case has a positive or negative residual and what the most overpredicted and underpredicted cases are. This is what the functions `most_overpredicted()` and `most_underpredicted()` achieve, each taking an `lm` object as input.
```{r}
# largest positive residual
most_underpredicted(df)
# largest negative residual
most_overpredicted(df)
```

The package does not include functions for plotting the cases. There are multiple, very useful packages such as the [olsrr](https://olsrr.rsquaredacademy.com/articles/residual_diagnostics.html) package that can be used for the easy visualization of residuals. 

---
title: "Classification as typical and deviant"
author: "Ingo Rohlfing"
date: "`r Sys.Date()`"
output: rmarkdown::html_vignette
vignette: >
  %\VignetteIndexEntry{Classification as typical and deviant}
  %\VignetteEngine{knitr::rmarkdown}
  %\VignetteEncoding{UTF-8}
---

```{r, include = FALSE}
knitr::opts_chunk$set(
  collapse = TRUE,
  comment = "#>"
)
```

```{r setup}
library(MMRcaseselection)
library(ggplot2) # for illustration of plot customization
```

The classification (designation, assignment) of cases as typical and deviant needs to be distinguished from the actual choice of cases that are of a specific type. The package offers two ways of classifying cases. First, one can designate cases using the *prediction interval* or the *standard deviation of the residuals*. Both classification techniques require an `lm` object as input. 

The approach using the prediction interval additionally requires the specification of the width of the interval. The % prediction interval represents the range of outcome values within which 95% of future outcome values are expected to fall in repeated samples. Following Rohlfing and Starke ([2013](https://doi.org/10.1111/spsr.12052)), a case is designated as typical if the observed outcome value is inside the interval and as deviant otherwise. 

Here is a simple example using the `mtcars` dataset and a 90% prediction interval (for no specific reason, the default value is 0.95). The output of the function is a dataframe with the observed and fitted outcome values, the upper and lower bound of the prediction interval and the case status based on the fitted value and the prediction interval that is specific to each case.
```{r}
df <- lm(mpg ~ disp + wt, data = mtcars)
pi_df <- predint(df, piwidth = 0.9)
head(pi_df)
```

The designation of cases based on the prediction interval can be visualized with the `predint_plot()` command. The graph plots observed vs. fitted outcome values together with the % prediction interval. The plot is a `gg` object and can be edited with the usual commands of the `ggplot2` package. 

The input into the plotting function has to be a dataframe produced with `predint()`. The color scheme is the [viridis color palette](https://CRAN.R-project.org/package=viridis) that is accustomed for colorblind readers and also works when printed in grey scale. If the viridis palette is not preferred, one can substitute it with any other `scale_color_*` scheme by adding it to the command.
```{r, fig.height = 6, fig.width = 6}
predint_plot(pi_df)
# Using scale_color_brewer() instead of the viridis palette.
predint_plot(pi_df) + scale_color_brewer()
```

An alternative classification technique following Lieberman ([2005](https://doi.org/10.1017/S0003055405051762)) uses the *standard deviation of the residuals*. The command for assigning cases and typical and deviant is `residstd()`. The specification of `stdshare` determines how large the share of the residual standard deviation should be to distinguish typical from deviant cases. The default value is 1 (for no specific reason).
```{r}
resid_df <- residstd(df, stdshare = 1.5)
head(resid_df)
```

To my knowledge, it has not been proposed to plot the case classification resulting from this technique. However, it can be done and is available in this package with the command `residstd_plot()`. The standard deviation of the residual is necessarily the same for all cases and is represented by a symmetric corridor (parallel grey lines) around the bisecting line in a plot of observed vs. fitted outcome values. The plot is a `gg` object that can be accustomed with `ggplot2` commands.
```{r, fig.width = 6, fig.height = 6}
residstd_plot(resid_df)
```

## Packages used in this vignette
-   base (R Core Team 2020)
-   ggplot2 (Wickham 2016)
-   grateful (Rodriguez-Sanchez 2018)

R Core Team. 2020. *R: A Language and Environment for Statistical
Computing*. Vienna, Austria: R Foundation for Statistical Computing.
<https://www.R-project.org/>.

Rodriguez-Sanchez, Francisco. 2018. *grateful: Facilitate Citation of R Packages*.
<https://github.com/Pakillo/grateful>.

Wickham, Hadley. 2016. *ggplot2: Elegant Graphics for Data Analysis*.
Springer-Verlag New York. <https://ggplot2.tidyverse.org>.

---
title: "Extreme case"
output: rmarkdown::html_vignette
vignette: >
  %\VignetteIndexEntry{Extreme case}
  %\VignetteEngine{knitr::rmarkdown}
  %\VignetteEncoding{UTF-8}
---

```{r, include = FALSE}
knitr::opts_chunk$set(
  collapse = TRUE,
  comment = "#>"
)
```

```{r setup}
library(MMRcaseselection)
```

The extreme case is selected with regard to the cases' values on an independent variable or the outcome. It is defined by the absolute difference between the case value on the chosen variable and the variable's mean. For example, for the outcome the *extremeness* of a case is defined as $|Y_i-\hat{Y}|$, with $i$ being the case index. The extreme case is the case with the maximum absolute difference. (see Seawright ([2016](https://doi.org/10.1177/0049124116643556))) Depending on the research question or substantive interest, one might be interested in the extreme case in the lower or upper range of a variable. Extremeness is then calculated with $Y_i-\hat{Y}$.

The `extreme_on_x()` and `extreme_on_y()` functions take an `lm` object as input and calculate the extremeness of all cases. For extremeness on an independent variable, one additionally needs to specify the variable of interest as a *character*. The output is a dataframe and cases are ordered by absolute extremeness in decreasing order. The dataframe also presents the extremeness values that show whether the case is extreme in the lower range of the variable (negative values) or the positive range (positive values).
```{r}
df <- lm(mpg ~ disp + wt, data = mtcars)
extreme_on_x(df, "wt")
```

The calculation of extremeness on the outcome only requires an `lm` object as input.
```{r}
df <- lm(mpg ~ disp + wt, data = mtcars)
Y_extreme <- extreme_on_y(df)
head(Y_extreme)
```

---
title: "Pathway case"
author: "Ingo Rohlfing"
date: "`r Sys.Date()`"
output: rmarkdown::html_vignette
vignette: >
  %\VignetteIndexEntry{Pathway case}
  %\VignetteEngine{knitr::rmarkdown}
  %\VignetteEncoding{UTF-8}
---

```{r, include = FALSE}
knitr::opts_chunk$set(
  collapse = TRUE,
  comment = "#>"
)
```

```{r setup}
library(MMRcaseselection)
```

The pathway was originally proposed by Gerring ([2007](https://doi.org/10.1177%2F0010414006290784)). He defines the *pathway value* of a case as  $|resid_{i_{reduced}}-resid_{i_{full}}|$ if it holds that $|resid_{i_{reduced}}| > |resid_{i_{full}}|$, where 'full' stands for the full regression model, 'reduced' for the model that lacks the pathway variable of theoretical interest and $i$ being a case index. Following Gerring, one should only choose among the cases meeting the requirement that $|resid_{i_{reduced}}| > |resid_{i_{full}}|$. In follow up research, Weller and Barnes ([2014](https://doi.org/10.1017/CBO9781139644501)) propose a different calculation of the pathway value, $|resid_{i_{reduced}}|-|resid_{i_{full}}|$, without specifying an additional requirement about the relationship between the full model residuals and reduced model residuals.

The function `pathway()` calculates both types of pathway values and requires the full regression model and the reduced regression model as input. Both models must be `lm` objects. The dataframe generated by the function contains all variables from the full model plus the following variables:

- `full_resid`: Residuals in full model
- `reduced_resid`: Residuals in reduced model
- `pathway_wb`: Pathway values as proposed by Weller and Barnes ($|resid_{i_{reduced}}|-|resid_{i_{full}}|$)
- `pathway_gvalue`: Pathway values as proposed by Gerring ($|resid_{i_{reduced}}-resid_{i_{full}}|$)
- `pathway_gstatus`: Binary character variable that is coded "yes" if $|resid_{reduced}| > |resid_{full}|$ is met and "no" otherwise.
```{r}
df_full <- lm(mpg ~ disp + wt, data = mtcars) # full model
df_reduced <- lm(mpg ~ wt, data = mtcars) # reduced model dropp 'disp' as pathway variable
pw_out <- pathway(df_full, df_reduced) # calculation of pathway variables
head(pw_out)
```

The visualization of pathway values is different from the presentation of ordinary residuals because two models are involved and an observed-vs-fitted plot is not meaningful. Following the approach by Weller and Barnes, the `pathway_xvr()` function plots the pathway values against the pathway variable. The option `pathwaytype = "pathway_wb` produces a plot for the Weller/Barnes values. The pathway variable is determined by the function and does not have to be specified. The plot is a `gg` object that can be customized with the usual `ggplot2` options.
```{r, fig.height = 6, fig.width = 6}
pathway_xvr(df_full, df_reduced, pathway_type = "pathway_wb")
```

The Gerring pathway values are plotted against the pathway variable if the option is `pathway_type = "pathway_gvalue". A color scheme is used to distinguish the cases that meet the pathway case requirement ("yes") from those that don't ("no"). 
```{r, fig.height = 6, fig.width = 6}
pathway_xvr(df_full, df_reduced, pathway_type = "pathway_gvalue")
```

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/classification.R
\name{predint}
\alias{predint}
\title{Classification of cases as typical and deviant using a prediction
interval.}
\usage{
predint(lmobject, piwidth = 0.95)
}
\arguments{
\item{lmobject}{Object generated with \code{\link[stats]{lm}}}

\item{piwidth}{Width of the prediction interval (default is 0.95).}
}
\value{
A dataframe with the observed outcome, fitted outcome,
upper and lower bound of the \% prediction interval and classification
of cases as typical or deviant.
}
\description{
Case are designated as typical (= well predicted) and deviant
(= badly predicted) based on the prediction interval. The x\% prediction
interval represents the range that we expect to include
x\% of outcome values in repeated samples. For example, a 95\%
prediction interval ranging from 0-5 conveys that 95\% of future outcome
values will be in the range of 0-5. If the observed outcome
is inside the prediction interval, the case is classified (or designated) as
 typical and as deviant otherwise.
}
\details{
Proposed by Rohlfing, Ingo and Peter Starke (2013):
Building on Solid Ground: Robust Case Selection in Multi-Method Research.
*Swiss Political Science Review* 19 (4): 492-512.
(\doi{10.1111/spsr.12052})
}
\examples{
df <- lm(mpg ~ disp + wt, data = mtcars)
predint(df, piwidth = 0.9)

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/classification.R
\name{predint_plot}
\alias{predint_plot}
\title{Plot of typical and deviant cases with prediction intervals}
\usage{
predint_plot(pred_df)
}
\arguments{
\item{pred_df}{A dataframe created with \code{\link{predint}}.}
}
\value{
A plot of the observed outcome against the fitted outcome with
prediction intervals and case classifications. Created with
\pkg{ggplot2}.
}
\description{
Presented in Rohlfing, Ingo and Peter Starke (2013):
Building on Solid Ground: Robust Case Selection in Multi-Method Research.
\emph{Swiss Political Science Review} 19 (4): 492-512.
(\doi{10.1111/spsr.12052})
}
\examples{
df <- lm(mpg ~ disp + wt, data = mtcars)
predint_status <- predint(df, piwidth = 0.9)
predint_plot(predint_status)

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/classification.R
\name{residstd}
\alias{residstd}
\title{Classification of cases as typical and deviant using the standard
deviation of the residuals.}
\usage{
residstd(lmobject, stdshare = 1)
}
\arguments{
\item{lmobject}{Object generated with \code{\link[stats]{lm}}}

\item{stdshare}{Share of standard deviation of residuals distinguishing
between typical and deviant cases (default is 1).}
}
\value{
A dataframe with the observed outcome, fitted outcome,
residual standard deviation and classification of cases as typical
or deviant.
}
\description{
The share of the standard deviation of the residuals is used to
designate cases as typical or deviant.
}
\details{
Proposed by Lieberman, Evan S. (2005): Nested Analysis as a Mixed-Method
Strategy for Comparative Research. \emph{American Political Science Review}
99 (3): 435-452. \doi{10.1017/S0003055405051762}.
}
\examples{
df <- lm(mpg ~ disp + wt, data = mtcars)
residstd(df, stdshare = 1)

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/extreme.R
\name{extreme_on_x}
\alias{extreme_on_x}
\title{Extremeness of cases on an independent variable}
\usage{
extreme_on_x(lmobject = NULL, ind_var = NULL)
}
\arguments{
\item{lmobject}{Object generated with \code{\link[stats]{lm}}}

\item{ind_var}{Independent variable for which extremeness values should
be calculated. Has to be entered as a character.}
}
\value{
A dataframe with

- all variables in the linear model,

- absolute extremeness (absolute value of difference between variable
score and mean value of variable),

- extremeness (difference between variable score and mean value of
variable), which can be useful when the direction of extremeness is relevant.

The rows are ordered in decreasing order of the absolute extreme values.
}
\description{
Extremeness of a case is calculated by the difference between a case's
value on the independent variable and the variable's mean value.
}
\details{
Calculating the absolute value of the difference between the cases' values
and the variable's mean value is proposed by Seawright, Jason (2016): The
Case for Selecting Cases That Are Deviant or Extreme on the Independent
Variable. \emph{Sociological Methods & Research} 45 (3): 493-525.
(\doi{10.1177/0049124116643556})
}
\examples{
df <- lm(mpg ~ disp + wt, data = mtcars)
extreme_on_x(df, "wt")

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/deviant.R
\name{most_underpredicted}
\alias{most_underpredicted}
\title{Identification of the most underpredicted case}
\usage{
most_underpredicted(lmobject)
}
\arguments{
\item{lmobject}{Object generated with \code{\link[stats]{lm}}}
}
\value{
The most underpredicted case with the largest positive residual
(the most positive residual).
}
\description{
The case with the largest positive difference between the observed
value and the predicted value on the outcome. Depending on the research
question, there might be a specific interest in the case for which the
model performs worst and yields a smaller predicted value.
}
\examples{
df <- lm(mpg ~ disp + wt, data = mtcars)
most_underpredicted(df)

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/deviant.R
\name{most_deviant}
\alias{most_deviant}
\title{Identification of the most deviant case}
\usage{
most_deviant(lmobject)
}
\arguments{
\item{lmobject}{Object generated with \code{\link[stats]{lm}}}
}
\value{
The most deviant case with the largest absolute
residual of all cases.
}
\description{
Identification of the most deviant case (= worst predicted case),
based on regression estimates.
}
\details{
Proposed by Seawright, Jason and John Gerring (2008):
Case Selection Techniques in Case Study Research: A Menu of
Qualitative and Quantitative Options. \emph{Political Research Quarterly}
61 (2): 294-308.
(\doi{10.1177/1065912907313077})
}
\examples{
df <- lm(mpg ~ disp + wt, data = mtcars)
most_deviant(df)
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/pathway.R
\name{pathway_xvr}
\alias{pathway_xvr}
\title{Plot of residuals against pathway variable}
\usage{
pathway_xvr(full_model, reduced_model, pathway_type)
}
\arguments{
\item{full_model}{Full model including covariate of interest
(= pathway variable)}

\item{reduced_model}{Reduced model excluding covariate of interest}

\item{pathway_type}{Type of pathway values. \code{pathway_wb} are
pathway values proposed by Weller and Barnes. \code{pathway_gvalue}
are values as calculated by Gerring.}
}
\value{
A plot of the chosen type of pathway values against the pathway
variable created with \pkg{ggplot2}.
}
\description{
Plot of residuals against pathway variable
}
\examples{
df_full <- lm(mpg ~ disp + wt, data = mtcars)
df_reduced <- lm(mpg ~ wt, data = mtcars)
pathway_xvr(df_full, df_reduced, pathway_type = "pathway_wb")

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/classification.R
\name{residstd_plot}
\alias{residstd_plot}
\title{Plot of typical and deviant cases based on residuals' standard deviation}
\usage{
residstd_plot(resid_df)
}
\arguments{
\item{resid_df}{A dataframe created with \code{\link{residstd}}.}
}
\value{
A plot of the observed outcome against the fitted outcome with
interval and case classifications. Created with \pkg{ggplot2}.
}
\description{
Plot of typical and deviant cases based on residuals' standard deviation
}
\examples{
df <- lm(mpg ~ disp + wt, data = mtcars)
residstd_status <- residstd(df, stdshare = 1)
residstd_plot(residstd_status)

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/pathway.R
\name{pathway}
\alias{pathway}
\title{Pathway case}
\usage{
pathway(full_model, reduced_model)
}
\arguments{
\item{full_model}{Full model including covariate of interest
(= pathway variable)}

\item{reduced_model}{Reduced model excluding covariate of interest}
}
\value{
A dataframe with

- all full model variables,

- full model residuals (\code{full_resid}),

- reduced model residuals (\code{reduced_resid}),

- pathway values following Weller/Barnes (\code{pathway_wb}),

- pathway values following Gerring (\code{pathway_gvalue}),

- variable showing whether Gerring's criterion for a pathway
case is met (\code{pathway_gstatus})
}
\description{
Calculation of pathway values, defined as the difference between residuals of
full model and reduced model lacking the pathway variable. The larger the
difference, the more a case qualifies as a pathway case suitable for the
analysis of mechanisms.
}
\details{
The difference between the absolute residuals of the full and reduced model
follows the approach developed by Weller and Barnes (2014): \emph{Finding
Pathways: Mixed-Method Research for Studying Causal Mechanisms.}
Cambridge: Cambridge University Press.
\doi{10.1017/CBO9781139644501}).

The calculation of the absolute difference between the full-model and
reduced-model residuals, given a case's reduced-model residual is larger
than its full-model residual, follows the proposal by
Gerring (2007): Is There a (Viable) Crucial-Case Method?
\emph{Comparative Political Studies} 40 (3): 231-253.
\doi{10.1177/0010414006290784})
}
\examples{
df_full <- lm(mpg ~ disp + wt, data = mtcars)
df_reduced <- lm(mpg ~ wt, data = mtcars)
pathway(df_full, df_reduced)

}
